@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2020,2021
+;;  Copyright (C) 2013-2024
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -26,6 +26,7 @@
   #:use-module (artanis fprm)
   #:use-module (artanis ssql)
   #:use-module (artanis session)
+  #:use-module (artanis cookie)
   #:use-module (artanis oht)
   #:use-module (artanis route)
   #:use-module (artanis page)
@@ -37,6 +38,7 @@
   #:use-module (artanis cache)
   #:use-module (artanis websocket)
   #:use-module (artanis lpc)
+  #:use-module (artanis security nss)
   #:use-module (artanis version)
   #:use-module (web server)
   #:use-module (srfi srfi-1)
@@ -98,7 +100,7 @@
                connect-db
                make-<connection>
                <connection>?
-               current-connection
+               get-conn-from-pool!
 
                ;; fprm module
                map-table-from-DB
@@ -137,6 +139,7 @@
                get-from-qstr
                get-referer
                params
+               get-sid-from-client-cookie
 
                ;; csv
                make-csv-reader
@@ -151,8 +154,6 @@
                scm->json-string
                json->scm
                json-string->scm
-               json-parser?
-               json-parser-port
 
                ;; upload
                mfd-simple-dump
@@ -187,7 +188,8 @@
 
                ;; version
                artanis-version)
-  #:export (result-ref
+  #:export (get-spawned-session-id
+            result-ref
             init-server
             form-tag
             label-tag
@@ -196,6 +198,9 @@
             div-tag
             run))
 
+(define* (get-spawned-session-id rc #:optional (sid "sid"))
+  (cookie-ref (car (rc-set-cookie rc)) sid))
+
 (define* (result-ref alst key #:key (decode? #t))
   (and=> (assoc-ref alst key) (if decode? uri-decode identity)))
 
@@ -222,48 +227,49 @@
   (sigaction SIGPIPE SIG_IGN) ; surpass SIGPIPE signal since we want to handle EPIPE by self
   (sigaction SIGINT (lambda (i)
                       (run-when-sigint-hook)
+                      (nss:pr-cleanup)
                       (format (artanis-current-output)
                               "~%Fare you well, your server is cold.~%")
                       (quit)))
-  (set! is-init-server-run? #t))
+  (is-init-server-run? #t))
 
 (define* (form-tag #:key (controller #f) (action #f) (method #f)
                    (class #f) (tag-class #f) (tag-id #f) (form-method "get"))
   (lambda tags
     (call-with-output-string
-      (lambda (port)
-        (format port "<form accept-charset='~a'" (get-conf '(server charset)))
-        (format port " action='~a'"
-                (call-with-output-string
-                  (lambda (port2)
-                    (display "/" port2)
-                    (and controller (format port2 "~a/" controller))
-                    (and action (format port2 "~a?" action))
-                    (and method (format port2 "method=~a" method))
-                    (and class (format port2 "&class=~a" class)))))
-        (format port " method='~a'" method)
-        (and tag-class (format port " class='~a'" tag-class))
-        (and tag-id (format port " id='~a'" tag-id))
-        (display ">\n" port)
-        (for-each (lambda (tag) (format port "~a~%" tag)) tags)
-        (format port "</form>~%")))))
+     (lambda (port)
+       (format port "<form accept-charset='~a'" (get-conf '(server charset)))
+       (format port " action='~a'"
+               (call-with-output-string
+                (lambda (port2)
+                  (display "/" port2)
+                  (and controller (format port2 "~a/" controller))
+                  (and action (format port2 "~a?" action))
+                  (and method (format port2 "method=~a" method))
+                  (and class (format port2 "&class=~a" class)))))
+       (format port " method='~a'" method)
+       (and tag-class (format port " class='~a'" tag-class))
+       (and tag-id (format port " id='~a'" tag-id))
+       (display ">\n" port)
+       (for-each (lambda (tag) (format port "~a~%" tag)) tags)
+       (format port "</form>~%")))))
 
 (define (make-general-tag tag)
   (lambda attrs
     (lambda (contents)
       (call-with-output-string
-        (lambda (port)
-          (format port "<~a" tag)
-          (let lp((next attrs))
-            (cond
-             ((null? next)
-              (display ">\n" port)
-              (display contents port)
-              (newline port)
-              (format port "</~a>" tag))
-             (else
-              (format port " ~a='~a'" (keyword->symbol (car next)) (cadr next))
-              (lp (cddr attrs))))))))))
+       (lambda (port)
+         (format port "<~a" tag)
+         (let lp((next attrs))
+           (cond
+            ((null? next)
+             (display ">\n" port)
+             (display contents port)
+             (newline port)
+             (format port "</~a>" tag))
+            (else
+             (format port " ~a='~a'" (keyword->symbol (car next)) (cadr next))
+             (lp (cddr attrs))))))))))
 
 (define label-tag (make-general-tag 'label))
 (define a-tag (make-general-tag 'a))
@@ -302,7 +308,7 @@
           (print-websocket-frame body)
           "<websocket-frame>"))
      (else (throw 'artanis-err 500 ->proper-body-display "Invalid body type `~a'!" body))))
-  (when (not is-init-server-run?)
+  (when (not (is-init-server-run?))
     (error "Sorry, but you have to run (init-server) in the begining of you main program!"))
   (and host (conf-set! '(host addr) host))
   (and port (conf-set! '(host port) port))
