@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2015,2017,2021
+;;  Copyright (C) 2015,2017,2021,2022
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -208,8 +208,8 @@
               (artanis utils))
  ;; Put whatever you want to be called before server initilization here
 
- (init-server)
-
+ (init-server #:statics '(png gif jpg jpeg ico html js json csv xml css woff woff2 ttf))
+ (add-to-load-path (string-append (current-toplevel) \"/lib\"))
  ;; Put whatever you want to be called before server running here
 ")
 
