@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2022
+;;  Copyright (C) 2013-2024
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -295,7 +295,7 @@
        (else sql)))))
 
 (define *exception-opts*
-  '(#:no-edit #:default #:comment #:storage))
+  '(#:no-edit #:default #:comment #:storage #:unsigned))
 (define (is-exception-opt? x)
   (memq x *exception-opts*))
 
@@ -411,6 +411,9 @@
                   (dump #f) (primary-keys '()))
     (let* ((types (map ->types defs))
            (pks (gen-primary-keys primary-keys))
+           (engine (if (eq? 'mysql (get-conf '(db dbd)))
+                       engine
+                       ""))
            (sql (case if-exists?
                   ((overwrite drop)
                    (table-drop! tname)
@@ -539,7 +542,7 @@
                   ;;        FROM table_name
                   ;;        ORDER BY column_name [ASC|DESC]
                   (condition "")
-                  ;; Conditions, for example: #:cond (where #:name "nala")
+                  ;; Conditions, e.g, #:condition (where #:name "nala")
                   (foreach '())
                   ;; #:foreach accepts an associative list according to this form:
                   ;; (column-name (val1 val2 val3 ...))
