@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2015,2017,2018
+;;  Copyright (C) 2015,2017,2018,2019,2022,2023
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -35,7 +35,8 @@
             rename-column
             remove-column
             add-index
-            remove-index))
+            remove-index
+            set-row))
 
 (define-syntax create-artanis-migration
   (lambda (x)
@@ -54,92 +55,120 @@
              #:use-module (artanis fprm))
            (define #,(datum->syntax x '*migrate-handlers*) '())
            (define (#,(datum->syntax x 'migrate-add!) cmd handler)
-             (set! *migrate-handlers*
-               (assoc-set! *migrate-handlers* cmd handler)))
+             (set! #,(datum->syntax x '*migrate-handlers*)
+                   (assoc-set! #,(datum->syntax x '*migrate-handlers*) cmd handler)))
            (define-syntax #,(datum->syntax x 'migrate-create)
              (syntax-rules ::: ()
                            ((_ do-migrate-create :::)
-                            (migrate-add! 'create (lambda () do-migrate-create :::)))))
+                            (#,(datum->syntax x 'migrate-add!)
+                             'create (lambda () do-migrate-create :::)))))
            (define-syntax #,(datum->syntax x 'migrate-up)
              (syntax-rules ::: ()
                            ((_ do-migrate-up :::)
-                            (migrate-add! 'up (lambda () do-migrate-up :::)))))
+                            (#,(datum->syntax x 'migrate-add!)
+                             'up (lambda () do-migrate-up :::)))))
            (define-syntax #,(datum->syntax x 'migrate-down)
              (syntax-rules ::: ()
                            ((_ do-migrate-down :::)
-                            (migrate-add! 'down (lambda () do-migrate-down :::)))))
+                            (#,(datum->syntax x 'migrate-add!)
+                             'down (lambda () do-migrate-down :::)))))
            (define-syntax #,(datum->syntax x 'migrate-change)
              (syntax-rules ::: ()
                            ((_ do-migrate-change :::)
-                            (migrate-add! 'change (lambda () do-migrate-change :::)))))
+                            (#,(datum->syntax x 'migrate-add!)
+                             'change (lambda () do-migrate-change :::)))))
            (define-public #,(datum->syntax x 'migrator)
              (lambda (cmd . args)
                (cond
-                ((assoc-ref *migrate-handlers* cmd)
+                ((assoc-ref #,(datum->syntax x '*migrate-handlers*) cmd)
                  => (lambda (h) (h)))
                 (else
                  (format (current-error-port) "[HINT] Did you remove `migrate-~a'?~%" cmd)
-                 (throw 'artanis-err 500 migrator "Migrate: Invalid cmd `~a'!"
+                 (throw 'artanis-err 500 #,(datum->syntax x 'migrator)
+                        "Migrate: Invalid cmd `~a'!"
                         cmd))))))))))
 
 (define (create-table name . fl)
-  (let ((raw ((@@ (artanis mvc model) parse-raw-fields) fl))
-        (mt (map-table-from-DB (current-connection))))
+  (let* ((raw ((@@ (artanis mvc model) parse-raw-fields) fl))
+         (conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
     (format (artanis-current-output) "Creating table `~a'......" name)
-    (mt 'try-create name raw)
+    (apply mt 'try-create name raw)
+    (recycle-DB-conn conn)
     (format (artanis-current-output) "Regenerating migration cache......")
-    (flush-to-migration-cache name fl)
     (display "DONE.\n" (artanis-current-output))))
 
 (define (change-table name cl)
-  (let ((mt (map-table-from-DB (current-connection))))
+  (let* ((conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
     (format (artanis-current-output) "Changing table `~a'......" name)
     (mt 'mod 'alter name cl)
+    (recycle-DB-conn conn)
     (display "DONE.\n" (artanis-current-output))))
 
 (define (drop-table name)
-  (let ((mt (map-table-from-DB (current-connection))))
+  (let* ((conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
     (format (artanis-current-output) "Dropping table `~a'.........." name)
     (mt 'drop name)
+    (recycle-DB-conn conn)
     (display "DONE.\n" (artanis-current-output))))
 
 (define (rename-table old new)
-  (let ((mt (map-table-from-DB (current-connection))))
+  (let* ((conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
     (format (artanis-current-output)
             "Renaming table `~a' to `~a'.........." old new)
     (mt 'mod 'rename old new)
+    (recycle-DB-conn conn)
     (display "DONE.\n" (artanis-current-output))))
 
 (define (add-column tname . cl)
-  (let ((mt (map-table-from-DB (current-connection))))
+  (let* ((conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
     (format (artanis-current-output)
             "Adding columns `(~{~a~^,~})' to table `~a'.........."
             (map car cl) tname)
     (apply mt 'mod tname 'add cl)
+    (recycle-DB-conn conn)
     (display "DONE.\n" (artanis-current-output))))
 
 (define (change-column tname col type)
-  (let ((mt (map-table-from-DB (current-connection))))
+  (let* ((conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
     (format (artanis-current-output)
             "Changing column `~a' of table `~a'.........."
             (list col type) tname)
     (mt 'mod 'alter tname col type)
+    (recycle-DB-conn conn)
     (display "DONE.\n" (artanis-current-output))))
 
 (define (rename-column tname oldcol newcol . type)
-  (let ((mt (map-table-from-DB (current-connection))))
+  (let* ((conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
     (format (artanis-current-output)
             "Renaming column `~a' to `~a' from `~a'.........."
             oldcol newcol tname)
     (apply mt 'mod 'alter tname oldcol newcol type)
+    (recycle-DB-conn conn)
     (display "DONE.\n" (artanis-current-output))))
 
 (define (remove-column tname cname)
-  (let ((mt (map-table-from-DB (current-connection))))
+  (let* ((conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
     (format (artanis-current-output)
             "Removing column `~a' in table `~a'.........."
             cname tname)
     (apply mt 'mod 'column-drop tname cname)
+    (recycle-DB-conn conn)
+    (display "DONE.\n" (artanis-current-output))))
+
+(define (set-row tname . fl)
+  (let* ((conn (get-conn-from-pool!))
+         (mt (map-table-from-DB conn)))
+    (format (artanis-current-output) "Setting row of table `~a'......" tname)
+    (apply mt 'set tname fl)
+    (recycle-DB-conn conn)
     (display "DONE.\n" (artanis-current-output))))
 
 (define (add-index)
