@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2015,2018
+;;  Copyright (C) 2015,2018,2019,2022
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -20,6 +20,7 @@
 (define-module (artanis mvc model)
   #:use-module (artanis utils)
   #:use-module (artanis env)
+  #:use-module (artanis config)
   #:use-module (artanis fprm)
   #:use-module (artanis db)
   #:use-module (artanis irregex)
@@ -64,6 +65,25 @@
       (lambda (opts kv)
         (list (cadr kv) opts))))))
 
+(define *db-specific-types*
+  '((mysql
+     (text . longtext)
+     (boolean . boolean))
+    (postgresql
+     (text . text)
+     (boolean . boolean))
+    (sqlite3
+     (text . text)
+     (boolean . integer))))
+
+(define (db-specific-type type)
+  (or
+   (assoc-ref
+    (assoc-ref *db-specific-types* (get-conf '(db dbd)))
+    type)
+   (throw 'artanis-err 500 db-specific-type
+          "Invalid DB specific type `~a'" type)))
+
 (define (general-field-handler name . opts)
   (define (get-maxlen lst) (get-kw-val #:maxlen lst))
   (define (get-diswidth lst) (get-kw-val #:diswidth lst))
@@ -79,16 +99,23 @@
     ((small-integer) `(smallint ,@(get-diswidth opts)))
     ;; 64 long integer
     ((big-integer) (list 'integer 64 opts))
-    ;; NOTE: No, we may not going to provide binary field, in Django, binary field could be
-    ;;       used to store data which is upto 4GB into DB. It is a bad design IMO.
+    ;; NOTE: No, we may not going to provide fix-sized binary field, in Django,
+    ;;       BinaryField could be used to store data which is upto 4GB into DB.
+    ;;       It is a bad design to store binary BLOB directly IMO.
     ;; Raw binary data
     ;; ((binary) (list 'longblob opts)) ; longblob for mysql
-    ((boolean) (list 'boolean opts))
+    ;; NOTE: Unlimited sized plain text field is still acceptable.
+    ;;       It's OK if you convert blob into base64 string to text field.
+    ;;       But remember that you pay what you choose.
+    ((text) (list (db-specific-type 'text) opts))
+    ((boolean) (list (db-specific-type 'boolean) opts))
     ;; Integer part is the total number of digits.
     ;; Fractional part is the number of digits following the decimal point.
     ((float) `(float ,@(get-integer-fractional-part opts)))
-    ((double) (pk "double"`(double ,@(get-integer-fractional-part opts))))
-    ((char-field) `(varchar ,@(get-maxlen opts)))))
+    ((double) `(double ,@(get-integer-fractional-part opts)))
+    ((char-field) `(varchar ,@(get-maxlen opts)))
+    ((date-field) (apply date-field-handler name opts))
+    (else `(,name ,opts))))
 
 (define (date-field-handler now . opts)
   (let ((new-opts
@@ -99,22 +126,6 @@
                  (else '())))))
     (list 'date (opts-add new-opts opts))))
 
-(define *model-field-handlers*
-  `((auto . ,general-field-handler)
-    (tiny-integer . ,general-field-handler)
-    (small-integer . ,general-field-handler)
-    (big-integer . ,general-field-handler)
-    (boolean . ,general-field-handler)
-    (float . ,general-field-handler)
-    (double . ,general-field-handler)
-    (char-field . ,general-field-handler)
-    (date-field . ,date-field-handler)))
-
-;; handler returns a list contains the valid field type in certain DBD
-(define (model-field-add! name handler)
-  (set! *model-field-handlers*
-    (assoc-set! *model-field-handlers* name handler)))
-
 ;; NOTE: Different from auto_now* in Django, we allow users pass new value
 ;;       even auto_now* was specified.
 ;; NOTE: #:default will overwrite #:auto-now or #:auto-now-once. So don't use
@@ -141,7 +152,7 @@
 
 (define-syntax-rule (check-field-value type r)
   (unless r
-    (throw 'artanis-err 500
+    (throw 'artanis-err 500 'check-field-value
            (format #f "~a check failed: ~a" type r))))
 
 (define (auto-field-validator v)
@@ -193,7 +204,7 @@
 
 (define (field-validator-add! type validator)
   (set! *field-validators*
-    (assoc-set! *field-validators* type validator)))
+        (assoc-set! *field-validators* type validator)))
 
 (define (general-field-validator v)
   ;; TODO: maybe do some security check?
@@ -247,30 +258,37 @@
 
 (define (parse-raw-fields lst)
   (define (->type type opts)
-    (let ((h (assoc-ref *model-field-handlers* type)))
-      (if h
-          (apply h type opts)
-          (throw 'artanis-err 500 parse-raw-fields
-                 "Invalid field type `~a'!" type))))
-  (let lp((next lst) (ret '()))
+    (apply general-field-handler type opts))
+  (let lp((next lst) (main '()) (options '()))
     (match next
-      (() (reverse ret))
+      (() `(,(reverse main) ,@(reverse options)))
       (((? keyword? k) v rest ...)
-       (lp rest (list v k ret)))
+       (lp rest main `(,v ,k ,@options)))
       ((((? symbol? name) (? symbol? type) . (or (? null? opts) ((opts ...)))) rest ...)
-       (lp rest (cons `(,name ,@(->type type opts)) ret)))
+       (lp rest (cons `(,name ,@(->type type opts)) main) options))
       (else (throw 'artanis-err 500 parse-raw-fields
                    "Invalid field definition!" next)))))
 
 ;; For example:
-;; (define-person
+;; (create-artanis-model
+;;   people
 ;;   (id auto (#:not-null #:primary-key))
 ;;   (first_name char-field (#:maxlen 30 #:not-null))
 ;;   (last_name char-field (#:maxlen 30 #:not-null)))
 (define-syntax create-artanis-model
   (lambda (x)
-    (syntax-case x ()
+    (syntax-case x (:deps)
       ((_ name) (identifier? #'name)
+       #`(begin
+           (define-module (app models name)
+             #:use-module (artanis artanis)
+             #:use-module (artanis utils)
+             #:use-module (artanis db)
+             #:use-module (artanis fprm))
+           (format (artanis-current-output)
+                   (WARN-TEXT "You have created model `~a' without any definition!~%")
+                   'name)))
+      ((_ name (:deps deps* ...) rest rest* ...) (identifier? #'name)
        #`(begin
            ;; NOTE: we have to encapsulate them to a module for protecting namespaces
            ;; NOTE: we're not going to imort (artanis env) directly to avoid revealing global
@@ -280,22 +298,24 @@
              #:use-module (artanis utils)
              #:use-module (artanis db)
              #:use-module (artanis fprm))
-           (define-syntax #,(datum->syntax #'name (symbol-append 'define- (syntax->datum #'name)))
-             (syntax-rules ::: ()
-                           ((_ rest rest* :::)
-                            (define-public #,(datum->syntax #'name (symbol-append '$ (syntax->datum #'name)))
-                              (let* ((raw (parse-raw-fields (list `rest `rest* :::)))
-                                     (mt (map-table-from-DB (current-connection)))
-                                     (meta (create-model-meta (list `rest `rest* :::))))
-                                (when (not (mt 'table-exists? name))
-                                  (format (artanis-current-output)
-                                          "Creating table `~a' defined in model......"
-                                          name)
-                                  (mt 'try-create name raw)
-                                  (format (artanis-current-output) "Done.~%"))
-                                (lambda (cmd . args)
-                                  (apply mt cmd 'name (fix-fields cmd args meta))))))))
-           (try-to-load-migrate-cache 'name))))))
+
+           (eval-when (expand load eval)
+             (when (not (null? (list 'deps* ...)))
+               (process-use-modules
+                (list
+                 (map (lambda (m) `(app models ,m))
+                      (list `deps* ...)))))
+
+             (define-public #,(datum->syntax #'name (symbol-append '$ (syntax->datum #'name)))
+               (let ((raw (parse-raw-fields (list `rest `rest* ...)))
+                     (mt (map-table-from-DB (get-conn-from-pool!))))
+                 (format (artanis-current-output)
+                         "Creating table `~a' defined in model~%......~%"
+                         'name)
+                 (apply mt 'try-create 'name raw)
+                 (format (artanis-current-output) "Done.~%")
+                 (lambda (cmd . args)
+                   (apply mt cmd 'name args))))))))))
 
 (define (gen-model-header name)
   (call-with-output-string
@@ -324,17 +344,16 @@
 (define-syntax-rule (scan-models) (scan-app-components 'models))
 
 (define (load-app-models)
-  (define toplevel (current-toplevel))
-  (display "Loading models...\n" (artanis-current-output))
-  (use-modules (artanis mvc model)) ; black magic to make Guile happy
-  (let ((cs (scan-models)))
-    (for-each (lambda (s)
-                (load (format #f "~a/app/models/~a.scm" toplevel s)))
-              cs)))
+  ;; NOTE:
+  ;; 1. For common cases, we don't need to load models in init stage, we can postpone it
+  ;;    when users need to.
+  ;; 2. For special cases, we can put stuffs here for them.
+  #t)
 
 ;; NOTE: id will be generated automatically, as primary-key.
 ;;       You may remove it to add your own primary-key.
 ;; NOTE: Each field will be set as #:not-null, modify it as you wish.
 (define (do-model-create name fields port)
+  (display "(import (artanis mvc model))\n" port)
   (display (gen-model-header name) port)
   )
