@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019
+;;  Copyright (C) 2013-2020,2024
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -18,10 +18,6 @@
 ;;  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (artanis utils)
-  #:use-module (artanis crypto md5)
-  #:use-module (artanis crypto sha-1)
-  #:use-module (artanis crypto sha-2)
-  #:use-module (artanis crypto base64)
   #:use-module (artanis tpl sxml)
   #:use-module (artanis config)
   #:use-module (artanis irregex)
@@ -47,20 +43,29 @@
   #:use-module ((rnrs)
                 #:select (get-bytevector-all
                           utf8->string put-bytevector
-                          bytevector-u8-ref string->utf8 bytevector-length
-                          make-bytevector bytevector-s32-native-ref bytevector?
-                          define-record-type record-rtd record-accessor
-                          get-string-all bitwise-ior))
+                          bytevector-u8-ref
+                          string->utf8
+                          bytevector-length
+                          make-bytevector
+                          bytevector-s32-native-ref
+                          bytevector?
+                          define-record-type
+                          record-rtd
+                          get-string-all
+                          bitwise-ior
+                          div
+                          get-u8
+                          get-bytevector-n
+                          bytevector->u8-list))
   #:export (regexp-split
             hash-keys cat bv-cat get-global-time sanitize-response
-            get-local-time string->md5 unsafe-random parse-date write-date
+            get-local-time unsafe-random parse-date write-date
             get-file-ext get-global-date get-local-date
             string-substitute nfx static-filename remote-info seconds-now local-time-stamp
             make-expires export-all-from-module!
             alist->hashtable expires->time-utc local-eval-string
             time-expired? valid-method? mmap munmap get-random-from-dev
-            string->byteslist string->sha-1 string->sha-224 string->sha-256 string->sha-384
-            string->sha-512 list-slice bv-slice uni-basename detect-type-name
+            string->byteslist list-slice bv-slice uni-basename detect-type-name
             checkout-the-path make-string-template guess-mime prepare-headers
             new-stack new-queue stack-slots queue-slots stack-pop! stack-push!
             stack-top stack-empty? queue-out! queue-in! queue-head queue-tail
@@ -71,7 +76,7 @@
             alist->klist alist->kblist is-hash-table-empty?
             symbol-downcase symbol-upcase normalize-column run-before-run!
             sxml->xml-string run-after-request! run-before-response! run-when-DB-init! run-when-sigint-hook
-            run-when-sigint! make-pipeline HTML-entities-replace eliminate-evil-HTML-entities
+            run-when-sigint! run-when-refresh! make-pipeline HTML-entities-replace eliminate-evil-HTML-entities
             generate-kv-from-post-qstr handle-proper-owner run-after-websocket-handshake!
             generate-data-url verify-ENTRY exclude-dbd
             draw-expander remove-ext scan-app-components cache-this-route!
@@ -85,7 +90,7 @@
             colorize-string WARN-TEXT ERROR-TEXT REASON-TEXT
             NOTIFY-TEXT STATUS-TEXT get-trigger get-family get-addr request-path
             response-keep-alive? request-keep-alive? file->bytevector
-            procedure-name->string proper-toplevel gen-content-length
+            procedure-name->string gen-content-length
             make-file-sender file-sender? file-sender-size file-sender-thunk
             get-string-all-with-detected-charset make-unstop-exception-handler
             artanis-log exception-from-client exception-from-server render-sys-page
@@ -94,7 +99,7 @@
             is-guile-compatible-server-core? positive-integer? negative-integer?
             io-exception:peer-is-shutdown? io-exception:out-of-memory?
             out-of-system-resources? allow-long-live-connection?
-            define-c-function ffi-binding free-JS-announcement)
+            free-JS-announcement gen-cache-file current-route-cache)
   #:re-export (the-environment
                utf8->string
                bytevector?
@@ -127,13 +132,23 @@
 
 ;; There's a famous rumor that 'urandom' is safer, so we pick it.
 (define* (get-random-from-dev #:key (length 8) (uppercase #f))
+  (define-syntax-rule (gen-for-even port)
+    (let ((bv (get-bytevector-n port (div length 2))))
+      (format #f "~{~2,'0x~}" (bytevector->u8-list bv))))
+
   (call-with-input-file "/dev/urandom"
     (lambda (port)
-      (let* ((bv ((@ (rnrs) get-bytevector-n) port length))
-             (str (format #f "~{~2,'0x~}" ((@ (rnrs) bytevector->u8-list) bv))))
-        (if uppercase
-            (string-upcase str)
-            str)))))
+      ;; generate token directly when length is even, otherwise, add the last byte with 8bit length.
+      (and=>
+       (cond
+        ((even? length) (gen-for-even port))
+        (else
+         (string-append (gen-for-even port)
+                        (format #f "~x" (logand #xf (get-u8 port))))))
+       (lambda (str)
+         (if uppercase
+             (string-upcase str)
+             str))))))
 
 (define-syntax-rule (local-eval-string str e)
   (local-eval
@@ -175,24 +190,24 @@
 ;; default time is #f, get current time
 (define* (get-global-time #:optional (time #f) (nsec 0))
   (call-with-output-string
-    (lambda (port)
-      ;; NOTE: (time-utc->data t 0) to get global time.
-      (write-date
-       (time-utc->date
-        (if time (make-time 'time-utc nsec time) (current-time))
-        0)
-       port))))
+   (lambda (port)
+     ;; NOTE: (time-utc->data t 0) to get global time.
+     (write-date
+      (time-utc->date
+       (if time (make-time 'time-utc nsec time) (current-time))
+       0)
+      port))))
 
 
 ;; default time is #f, get current time
 (define* (get-local-time #:optional (time #f) (nsec 0))
   (call-with-output-string
-    (lambda (port)
-      ;; NOTE: (time-utc->data t) to get local time.
-      (write-date
-       (time-utc->date
-        (if time (make-time 'time-utc nsec time) (current-time)))
-       port))))
+   (lambda (port)
+     ;; NOTE: (time-utc->data t) to get local time.
+     (write-date
+      (time-utc->date
+       (if time (make-time 'time-utc nsec time) (current-time)))
+      port))))
 
 (define* (regexp-split regex str #:optional (flags 0))
   (let ((ret (fold-matches
@@ -272,8 +287,8 @@
 
 (define-syntax-rule (static-filename path)
   (if (current-toplevel)
-      (format #f "~a/~a" (current-toplevel) path)
-      (format #f "./~a" path)))
+      (format #f "~a/pub/~a" (current-toplevel) path)
+      (format #f "./pub/~a" path)))
 
 (define-syntax-rule (request-ip req)
   ;; TODO: support AF_INET6 in the future
@@ -378,66 +393,6 @@
       (lp (cons (string->number (substring/shared str i (+ i step)) base) ret) (1+ i)))
      (else (lp ret (1+ i))))))
 
-(define (string->md5 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-1
-                          "need string or bytevector!" str/bv)))))
-    (md5->string (md5 in))))
-
-(define (string->sha-1 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-1
-                          "need string or bytevector!" str/bv)))))
-    (sha-1->string (sha-1 in))))
-
-(define (string->sha-224 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-224
-                          "need string or bytevector!" str/bv)))))
-    (sha-224->string (sha-224 in))))
-
-(define (string->sha-256 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-256
-                          "need string or bytevector!" str/bv)))))
-    (sha-256->string (sha-256 in))))
-
-(define (string->sha-384 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-384
-                          "need string or bytevector!" str/bv)))))
-    (sha-384->string (sha-384 in))))
-
-(define (string->sha-512 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-512
-                          "need string or bytevector!" str/bv)))))
-    (sha-512->string (sha-512 in))))
-
 (define-syntax list-slice
   (syntax-rules (:)
     ((_ ll lo : hi)
@@ -729,8 +684,8 @@
 
 (define* (sxml->xml-string sxml #:key (escape? #f))
   (call-with-output-string
-    (lambda (port)
-      (sxml->xml sxml port escape?))))
+   (lambda (port)
+     (sxml->xml sxml port escape?))))
 
 (define (run-after-request! proc)
   (add-hook! *after-request-hook* proc))
@@ -753,6 +708,9 @@
 (define (run-when-sigint-hook)
   (run-hook *when-sigint-hook*))
 
+(define (run-when-refresh! proc)
+  (add-hook! *refresh-hook* proc))
+
 ;; NOTE: For `pipeline' methodology, please read my post:
 ;; http://nalaginrut.com/archives/2014/04/25/oba-pipeline-style%21
 (define (make-pipeline . procs)
@@ -767,28 +725,28 @@
        ((= n 3) (list->string (reverse! ret)))
        (else (lp (1+ n) (cons (read-char port) ret))))))
   (call-with-output-string
-    (lambda (out)
-      (let lp((c (peek-char in)))
-        (cond
-         ((eof-object? c) #t)
-         ((hit? c)
-          => (lambda (str)
-               (display str out)
-               (read-char in)
-               (lp (peek-char in))))
-         ((char=? c #\%)
-          (let* ((s (get-estr in))
-                 (e (hit? s)))
-            (if e
-                (display e out)
-                (display s out))
-            (lp (peek-char in))))
-         (else
-          (display (read-char in) out)
-          (lp (peek-char in))))))))
+   (lambda (out)
+     (let lp((c (peek-char in)))
+       (cond
+        ((eof-object? c) #t)
+        ((hit? c)
+         => (lambda (str)
+              (display str out)
+              (read-char in)
+              (lp (peek-char in))))
+        ((char=? c #\%)
+         (let* ((s (get-estr in))
+                (e (hit? s)))
+           (if e
+               (display e out)
+               (display s out))
+           (lp (peek-char in))))
+        (else
+         (display (read-char in) out)
+         (lp (peek-char in))))))))
 
 (define *terrible-HTML-entities*
-  '((#\< . "&lt;") (#\> . "&gt;") (#\& . "&amp;") (#\" . "&quot;")
+  '((#\< . "&lt;") (#\> . "&gt;") (#\& . "&amp;") (#\" . "&quot;") (#\sp . "&nbsp;")
     ("%3C" . "&lt;") ("%3E" . "&gt;") ("%26" . "&amp;") ("%22" . "&quot;")))
 ;; NOTE: cooked for anti-XSS.
 (define (eliminate-evil-HTML-entities content)
@@ -804,9 +762,15 @@
                    "Fatal! Can't be here `~a'!" lst))))
   (define (-> x)
     (string-trim-both x (lambda (c) (member c '(#\sp #\: #\return)))))
+  (define (fix x)
+    (cond
+     ((string? x) x)
+     ((bytevector? x) (utf8->string x))
+     (else (throw 'artanis-err 500 generate-kv-from-post-qstr
+                  "Fatal! Can't be here `~a'!" x))))
   (map (lambda (x)
          (%convert (map -> (string-split (cook x) #\=))))
-       (string-split (utf8->string body) #\&)))
+       (string-split (fix body) #\&)))
 
 ;; NOTE: We accept warnings, which means if warnings occurred, it'll be 200(OK) anyway, but
 ;;       Artanis will throw warnings in the server-side log.
@@ -853,7 +817,7 @@
       (else (throw 'artanis-err 500 generate-data-url
                    "Invalid MIME! Should be symbol or string"
                    mime))))
-  (let ((b64 (base64-encode bv/str)))
+  (let ((b64 (nss:base64-encode bv/str)))
     (string-concatenate (list "data:" (->mime) (->crypto) (->charset) "," b64))))
 
 (define (verify-ENTRY entry)
@@ -898,11 +862,16 @@
   (define (write-header port)
     (format port ";; Do not touch anything!!!~%")
     (format port ";; All things here should be automatically handled properly!!!~%"))
-  (define route-cache (string-append (current-toplevel) "/tmp/cache/route.cache"))
+  (define cache-dir (format #f "~a/cache" (current-tmp)))
+  (define route-cache (format #f "~a/cache/route.cache" (current-tmp)))
   (when (not (file-exists? route-cache))
     (format (artanis-current-output) "Regenerating route cache ...~%")
-    (call-with-output-file route-cache
-      (lambda (port) (write-header port) (write '() port))))
+    (when (not (file-exists? cache-dir))
+      (mkdir cache-dir))
+    (let ((port (open-file route-cache "w")))
+      (write-header port)
+      (write '() port)
+      (close port)))
   (when (and url meta)
     (let ((rl (call-with-input-file route-cache read)))
       (delete-file route-cache)
@@ -922,10 +891,13 @@
                ))
           (flock port LOCK_UN))))))
 
+(define (current-route-cache)
+  (format #f "~a/.route" (current-tmp)))
+
 (define (dump-route-from-cache)
   (define toplevel (current-toplevel))
-  (define route-cache (string-append toplevel "/tmp/cache/route.cache"))
-  (define route (string-append toplevel "/.route"))
+  (define route-cache (format #f "~a/cache/route.cache" (current-tmp)))
+  (define route (current-route-cache))
   (define (load-customized-router)
     (let ((croute (string-append toplevel "conf/route")))
       (cond
@@ -993,9 +965,9 @@
 
 (define (subbv->string bv encoding start len)
   (call-with-output-string
-    (lambda (port)
-      (set-port-encoding! port encoding)
-      (put-bytevector port bv start len))))
+   (lambda (port)
+     (set-port-encoding! port encoding)
+     (put-bytevector port bv start len))))
 
 (define* (bv-u8-index bv u8 #:optional (time 1))
   (let ((len (bytevector-length bv)))
@@ -1114,18 +1086,18 @@
           (catch 'interrupt
             (lambda ()
               (dynamic-wind
-                (lambda ()
-                  (set! handler
-                    (sigaction SIGINT (lambda (sig)
-                                        (run-when-sigint-hook)
-                                        (throw 'interrupt)))))
-                thunk
-                (lambda ()
-                  (if handler
-                      ;; restore Scheme handler, SIG_IGN or SIG_DFL.
-                      (sigaction SIGINT (car handler) (cdr handler))
-                      ;; restore original C handler.
-                      (sigaction SIGINT #f)))))
+                  (lambda ()
+                    (set! handler
+                          (sigaction SIGINT (lambda (sig)
+                                              (run-when-sigint-hook)
+                                              (throw 'interrupt)))))
+                  thunk
+                  (lambda ()
+                    (if handler
+                        ;; restore Scheme handler, SIG_IGN or SIG_DFL.
+                        (sigaction SIGINT (car handler) (cdr handler))
+                        ;; restore original C handler.
+                        (sigaction SIGINT #f)))))
             (lambda (k . _) (handler-thunk)))))))
 
 (define-syntax-rule (define-box-type name)
@@ -1135,7 +1107,7 @@
   (list (symbol-append 'make- bt) v))
 
 (define-syntax-rule (box-type-treasure t)
-  (record-accessor (record-rtd t) 0))
+  ((@ (rnrs) record-accessor) (record-rtd t) 0))
 
 (define-syntax-rule (unbox-type t)
   (let ((treasure-getter (box-type-treasure t)))
@@ -1331,9 +1303,6 @@
 (define (procedure-name->string proc)
   (symbol->string (procedure-name proc)))
 
-(define-syntax-rule (proper-toplevel)
-  (or (current-toplevel) ""))
-
 (define-record-type file-sender
   (fields size thunk))
 
@@ -1584,40 +1553,6 @@
 (define (allow-long-live-connection?)
   (> (get-conf '(server timeout)) 0))
 
-(define %%clib (make-parameter (dynamic-link)))
-
-(define-syntax define-c-function
-  (lambda (x)
-    (syntax-case x ()
-      ((_ type name)
-       #`(module-define!
-          (current-module)
-          '#,(datum->syntax #'name (symbol-append '% (syntax->datum #'name)))
-          (pointer->procedure type
-                              (dynamic-func (symbol->string 'name) (%%clib))
-                              '()
-                              #:return-errno? #t)))
-      ((_ type name (para ...))
-       #`(module-define!
-          (current-module)
-          '#,(datum->syntax #'name (symbol-append '% (syntax->datum #'name)))
-          (pointer->procedure type
-                              (dynamic-func (symbol->string 'name) (%%clib))
-                              (list para ...)
-                              #:return-errno? #t))))))
-
-(define-syntax ffi-binding
-  (syntax-rules ()
-    ((_ () body ...)
-     (begin
-       body ...
-       #t))
-    ((_ libname body ...)
-     (parameterize ((%%clib (dynamic-link libname)))
-       body ...
-       #t))))
-
-
 ;; Story: When I released GNU Artanis-0.2.1, RMS had asked me if I can
 ;;        support LibreJS for freeing Javascript code in the generated code.
 ;;        I promised I will. This took almost one year since I was very
@@ -1665,3 +1600,13 @@
         */
     </script>
 ")
+
+(define (gen-cache-file path)
+  (define-syntax-rule (-> str)
+    (string-trim-both
+     (irregex-replace/all "[/]+" str "_")
+     (lambda (x) (memv x '(#\sp #\_)))))
+  (let ((p (-> path)))
+    (if (string-null? p)
+        (format #f "~a/cache/index.html" (current-tmp))
+        (format #f "~a/cache/~a.html" (current-tmp) (-> path)))))
